# coding: utf-8

"""
collections.OrderedDict is a new class not supported by PyYAML (issue 83 by Frazer McLean)

This is now so integrated in Python that it can be mapped to !!omap

"""

import pytest    # NOQA
import ruamel.yaml   # NOQA
from collections import OrderedDict


from roundtrip import round_trip, dedent, round_trip_load, round_trip_dump  # NOQA


class TestOrderedDict:
    def test_ordereddict(self):
        assert ruamel.yaml.dump(OrderedDict()) == '!!omap []\n'
